// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/18AI64SSC750K/samples/irq/irq_main.c $
// $Rev: 47254 $
// $Date: 2020-05-18 17:44:19 -0500 (Mon, 18 May 2020) $

// 18AI64SSC750K: Sample Application: source file

#include "main.h"



//*****************************************************************************
static int _dma(int fd)
{
	char	buf[1024];
	s32		enable_a;
	s32		enable_b;
	int		errs	= 0;
	s32		fsamp	= 20000;
	s32		nrate_a;
	s32		nrate_b;
	s32		qty		= 64;
	s32		src;
	s32		src_b;
	int		status;

	errs	+= ai64ssc750k_query(fd, -1, 0, AI64SSC750K_QUERY_CHANNEL_QTY, &qty);

	errs	+= ai64ssc750k_initialize		(fd, -1, 0);
	errs	+= ai64ssc750k_rx_io_mode		(fd, -1, 0, GSC_IO_MODE_BMDMA,					NULL);
	errs	+= ai64ssc750k_rx_io_overflow	(fd, -1, 0, AI64SSC750K_IO_OVERFLOW_IGNORE,		NULL);
	errs	+= ai64ssc750k_rx_io_timeout	(fd, -1, 0, AI64SSC750K_IO_TIMEOUT_DEFAULT,		NULL);
	errs	+= ai64ssc750k_rx_io_underflow	(fd, -1, 0, AI64SSC750K_IO_UNDERFLOW_IGNORE,	NULL);
	errs	+= ai64ssc750k_ai_range			(fd, -1, 0, AI64SSC750K_AI_RANGE_5V,			NULL);
	errs	+= ai64ssc750k_ai_buf_thr_lvl	(fd, -1, 0, 250000,								NULL);
	errs	+= ai64ssc750k_ai_mode			(fd, -1, 0, AI64SSC750K_AI_MODE_SINGLE,			NULL);
	errs	+= ai64ssc750k_burst_size		(fd, -1, 0, 1,									NULL);
	errs	+= ai64ssc750k_burst_enable		(fd, -1, 0, AI64SSC750K_BURST_ENABLE_NO,		NULL);
	errs	+= ai64ssc750k_data_packing		(fd, -1, 0, AI64SSC750K_DATA_PACKING_DISABLE,	NULL);
	errs	+= ai64ssc750k_fsamp_ai_compute	(fd, -1, 0, fsamp, &src, &src_b, &nrate_a, &nrate_b, &enable_a, &enable_b, NULL);
	errs	+= ai64ssc750k_samp_clk_src		(fd, -1, 0, src,								NULL);
	errs	+= ai64ssc750k_rbg_clk_src		(fd, -1, 0, src_b,								NULL);
	errs	+= ai64ssc750k_rag_nrate		(fd, -1, 0, nrate_a,							NULL);
	errs	+= ai64ssc750k_rbg_nrate		(fd, -1, 0, nrate_b,							NULL);
	errs	+= ai64ssc750k_rag_enable		(fd, -1, 0, enable_a,							NULL);
	errs	+= ai64ssc750k_rbg_enable		(fd, -1, 0, enable_b,							NULL);
	errs	+= ai64ssc750k_scan_marker		(fd, -1, 0, AI64SSC750K_SCAN_MARKER_DISABLE,	NULL);
	errs	+= ai64ssc750k_clocking_enable	(fd, -1, 0, AI64SSC750K_CLOCKING_ENABLE_YES,	NULL);

	status	= ai64ssc750k_read(fd, buf, sizeof(buf));

	if (status == -1)
	{
		errs++;
		printf("ERROR: ai64ssc750k_read() failure, errno = %d\n", errno);
	}

	return(errs);
}




//*****************************************************************************
static int _irq_main_dma_test(int fd)
{
	int	errs;

	gsc_label("DMA0/DMA1");
	errs	= irq_wait_test(fd, GSC_WAIT_MAIN_DMA0 | GSC_WAIT_MAIN_DMA1, 0, 0, _dma);
	return(errs);
}



//*****************************************************************************
static int _gsc(int fd)
{
	int	errs;

	errs	= ai64ssc750k_initialize(fd, -1, 0);
	return(errs);
}



//*****************************************************************************
static int _irq_main_gsc_test(int fd)
{
	int	errs;

	gsc_label("GSC");
	errs	= irq_wait_test(fd, GSC_WAIT_MAIN_GSC, 0, 0, _gsc);
	return(errs);
}



//*****************************************************************************
static int _pci(int fd)
{
	int	errs;

	errs	= ai64ssc750k_initialize(fd, -1, 0);
	return(errs);
}



//*****************************************************************************
static int _irq_main_pci_test(int fd)
{
	int	errs;

	gsc_label("PCI");
	errs	= irq_wait_test(fd, GSC_WAIT_MAIN_PCI, 0, 0, _pci);
	return(errs);
}



//*****************************************************************************
int irq_main_test(int fd)
{
	int	errs	= 0;

	gsc_label("IRQ MAIN");
	printf("\n");
	gsc_label_level_inc();

	errs	+= _irq_main_dma_test(fd);
	errs	+= _irq_main_gsc_test(fd);
	errs	+= _irq_main_pci_test(fd);

	gsc_label_level_dec();
	return(errs);
}


