// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/18AI64SSC750K/samples/savedata/perform.c $
// $Rev: 53328 $
// $Date: 2023-07-05 16:15:16 -0500 (Wed, 05 Jul 2023) $

// 18AI64SSC750K: Sample Application: source file

#include "main.h"



// macros *********************************************************************

#define	_1M				(1024L * 1024L)



// variables ******************************************************************

static	u32	_buffer[_1M];



//*****************************************************************************
static int _channels(const args_t* args, s32* chans)
{
	s32		active;
	char	buf[64];
	int		errs	= 0;
	s32		first	= -1;
	s32		last	= -1;
	s32		single	= -1;
	int		ret;

	gsc_label("Input Channels");

	active	= -1;
	ret		= ai64ssc750k_ioctl(args->fd, AI64SSC750K_IOCTL_CHAN_ACTIVE,	&active	);
	errs	+= ret ? 1 : 0;
	ret		= ai64ssc750k_ioctl(args->fd, AI64SSC750K_IOCTL_CHAN_FIRST,	&first	);
	errs	+= ret ? 1 : 0;
	ret		= ai64ssc750k_ioctl(args->fd, AI64SSC750K_IOCTL_CHAN_LAST,	&last	);
	errs	+= ret ? 1 : 0;
	ret		= ai64ssc750k_ioctl(args->fd, AI64SSC750K_IOCTL_CHAN_SINGLE,	&single	);
	errs	+= ret ? 1 : 0;

	if (errs == 0)
	{
		switch (active)
		{
			default:

				errs++;
				printf("FAIL <---  (invalid ACTIVE selection: %ld)\n", (long) active);
				break;

			case AI64SSC750K_CHAN_ACTIVE_SINGLE:

				chans[0]	= 1;
				sprintf(buf, "# %ld", (long) single);
				break;

			case AI64SSC750K_CHAN_ACTIVE_0_1:

				chans[0]	= 2;
				strcpy(buf, "#s 0-1");
				break;

			case AI64SSC750K_CHAN_ACTIVE_0_3:

				chans[0]	= 4;
				strcpy(buf, "#s 0-3");
				break;

			case AI64SSC750K_CHAN_ACTIVE_0_7:

				chans[0]	= 8;
				strcpy(buf, "#s 0-7");
				break;

			case AI64SSC750K_CHAN_ACTIVE_0_15:

				chans[0]	= 16;
				strcpy(buf, "#s 0-15");
				break;

			case AI64SSC750K_CHAN_ACTIVE_0_31:

				chans[0]	= 32;
				strcpy(buf, "#s 0-31");
				break;

			case AI64SSC750K_CHAN_ACTIVE_0_63:

				chans[0]	= 64;
				strcpy(buf, "#s 0-63");
				break;

			case AI64SSC750K_CHAN_ACTIVE_RANGE:

				if (first > last)
					first	= last;

				chans[0]	= last - first + 1;

				if (first == last)
					sprintf(buf, "# %ld", (long) first);
				else
					sprintf(buf, "#s %ld-%ld", (long) first, (long) last);

				break;
		}

		if (errs == 0)
		{
			printf(	"%ld Channel%s  (%s)\n",
					(long) chans[0],
					(chans[0] == 1) ? "" : "s",
					buf);
		}
	}

	return(errs);
}



//*****************************************************************************
static int _read_data(const args_t* args, int chans)
{
	char	buf[80];
	int		errs	= 0;
	int		get;
	int		got;
	int		repeat;

	if (args->data_pack == AI64SSC750K_DATA_PACKING_DISABLE)
		get	= chans;
	else if (args->scan_marker == AI64SSC750K_SCAN_MARKER_DISABLE)
		get	= chans + (chans % 2);
	else
		get	= (chans + (chans % 2)) + 2;

	get	= ((SIZEOF_ARRAY(_buffer) / get) * get) * 4;

	for (repeat = 0; (errs == 0) && (repeat < args->repeat);)
	{
		repeat++;
		strcpy(buf, "Reading");

		if (args->repeat > 1)
		{
			strcat(buf, " #");
			gsc_label_long_comma_buf(repeat, buf + strlen(buf));
		}

		gsc_label(buf);

		got	= ai64ssc750k_read(args->fd, _buffer, get);

		if (got < 0)
		{
			errs	= 1;
			printf("FAIL <---  (read error: %d)\n", got);
			break;
		}

		if (got != get)
		{
			errs	= 1;
			printf(	"FAIL <---  (got %ld bytes, requested %ld)\n",
					(long) got,
					(long) get);
			break;
		}

		printf("PASS  (");
		gsc_label_long_comma(get / 4);
		printf(" samples)\n");
	}

	return(errs);
}



//*****************************************************************************
static int _save_eol(const char* name, FILE* file)
{
	int	errs	= 0;
	int	i;

	i	= fprintf(file, "\r\n");

	if (i != 2)
	{
		printf("FAIL <---  (fprintf() failure to %s)\n", name);
		errs	= 1;
	}

	return(errs);
}



//*****************************************************************************
static int _save_channel_value(const args_t* args, const char* name, FILE* file, u32 value, int chan)
{
	char	buf[32];
	int		errs	= 0;
	int		i;
	int		len;

	if (args->data_pack == AI64SSC750K_DATA_PACKING_ENABLE)
	{
		if (chan < 0)
			sprintf(buf, "  %08lX", (long) value);	// scan marker
		else if (args->decimal)
			sprintf(buf, "  %5ld", (long) value);
		else
			sprintf(buf, "  %04lX", (long) value);
	}
	else if (args->data_width == AI64SSC750K_DATA_WIDTH_16_BITS)
	{
		if (args->tag_strip)
			value	&= 0xFFFF;

		if (args->decimal)
			sprintf(buf, "  %5ld", (long) value);
		else
			sprintf(buf, "  %05lX", (long) value);
	}
	else
	{
		if (args->tag_strip)
			value	&= 0x3FFFF;

		if (args->decimal)
			sprintf(buf, "  %8ld", (long) value);
		else
			sprintf(buf, "  %05lX", (long) value);
	}

	len	= strlen(buf);
	i	= fprintf(file, "%s", buf);

	if (i != len)
	{
		printf("FAIL <---  (fprintf() failure to %s)\n", name);
		errs	= 1;
	}

	return(errs);
}



//*****************************************************************************
static int _save_value(const args_t* args, const char* name, FILE* file, int chans, int index)
{
	int			ch_div;
	static int	chan;
	int			errs	= 0;
	s32			tag;
	s32			value	= _buffer[index];
	int			wd_index;

	if (args->data_pack == AI64SSC750K_DATA_PACKING_DISABLE)
	{
		if (args->data_width == AI64SSC750K_DATA_WIDTH_18_BITS)
			tag		= 0x40000;
		else
			tag		= 0x10000;

		if ((index > 0) && ((chans == 1) || (value & tag)))
			errs	+= _save_eol(name, file);

		errs	+= _save_channel_value(args, name, file, value, 0);
	}
	else if (args->scan_marker == AI64SSC750K_SCAN_MARKER_DISABLE)
	{
		ch_div		= chans + (chans % 2);	// round up to the next even number
		ch_div		/= 2;
		wd_index	= index % ch_div;
		chan		= wd_index * 2;

		if ((index > 0) && (chan == 0))
			errs	+= _save_eol(name, file);

		errs	+= _save_channel_value(args, name, file, (u16) (value & 0xFFFF), chan);

		chan++;

		if (chan < chans)
			errs	+= _save_channel_value(args, name, file, (u16) (value >> 16), chan);
	}
	else
	{
		if (index == 0)
			chan	= -1;

		if (value == args->scan_mark)
		{
			if (index > 0)
				errs	+= _save_eol(name, file);

			chan	= -1;
			errs	+= _save_channel_value(args, name, file, value, chan);
		}
		else
		{
			chan++;
			errs	+= _save_channel_value(args, name, file, (u16) (value & 0xFFFF), chan);
			chan++;


			if (chan < chans)
				errs	+= _save_channel_value(args, name, file, (u16) (value >> 16), chan);
		}
	}

	return(errs);
}



//*****************************************************************************
static int _save_data(const args_t* args, int chans, int errs)
{
	char		buf[64];
	FILE*		file;
	long		index;
	const char*	name	= "data.txt";
	int			samples	= SIZEOF_ARRAY(_buffer);

	strcpy(buf, "Saving");

	if (args->force_save)
	{
		errs	= 0;
		strcat(buf, " (Forced)");
	}

	gsc_label(buf);

	for (;;)
	{
		if ((errs) && (args->force_save == 0))
		{
			printf("SKIPPED  (errors)\n");
			errs	= 0;
			break;
		}

		file	= fopen(name, "w+b");

		if (file == NULL)
		{
			printf("FAIL <---  (unable to create %s)\n", name);
			errs	= 1;
			break;
		}

		for (index = 0; index < samples; index++)
			errs	+= _save_value(args, name, file, chans, index);

		fclose(file);

		if (errs == 0)
			printf("PASS  (%s)\n", name);

		break;
	}

	return(errs);
}



//*****************************************************************************
static int _config(const args_t* args)
{
	s32	chan_active	= args->chan_active;
	s32	chan_first	= args->chan_first;
	s32	chan_last	= args->chan_last;
	s32	chan_single	= args->chan_single;
	s32	chans		= 32;
	int	err;
	int	errs		= 0;

	gsc_label("Configure");
	printf("\n");
	gsc_label_level_inc();

	errs	+= ai64ssc750k_query(args->fd, -1, 1, AI64SSC750K_QUERY_CHANNEL_QTY, &chans);

	if (chan_single >= chans)
		chan_single	= chans - 1;

	if (chan_first >= chans)
		chan_first	= chans - 1;

	if (chan_last >= chans)
		chan_last	= chans - 1;

	if (chan_first > chan_last)
		chan_first	= chan_last;

	gsc_label("Basic Configuration");
	err		= ai64ssc750k_config_ai(args->fd, -1, 0, args->fsamp);
	errs	+= err;
	printf("%s\n", err ? "FAIL <---" : "PASS");

	errs	+= ai64ssc750k_rx_io_mode		(args->fd, -1, 1, args->io_mode, NULL);
	errs	+= ai64ssc750k_rx_io_timeout	(args->fd, -1, 1, 60, NULL);
	errs	+= ai64ssc750k_ai_mode			(args->fd, -1, 1, args->ai_mode, NULL);
	errs	+= ai64ssc750k_data_format		(args->fd, -1, 1, args->data_format, NULL);
	errs	+= ai64ssc750k_chan_active		(args->fd, -1, 1, chan_active, NULL);
	errs	+= ai64ssc750k_chan_single		(args->fd, -1, 1, chan_single, NULL);
	errs	+= ai64ssc750k_chan_first		(args->fd, -1, 1, chan_first, NULL);
	errs	+= ai64ssc750k_chan_last		(args->fd, -1, 1, chan_last, NULL);
	errs	+= ai64ssc750k_data_packing		(args->fd, -1, 1, args->data_pack, NULL);
	errs	+= ai64ssc750k_data_width		(args->fd, -1, 1, args->data_width, NULL);
	errs	+= ai64ssc750k_scan_marker		(args->fd, -1, 1, args->scan_marker, NULL);
	errs	+= ai64ssc750k_scan_marker_set	(args->fd, -1, 1, args->scan_mark);
	errs	+= ai64ssc750k_query			(args->fd, -1, 1, AI64SSC750K_QUERY_V_RANGE_HIGH, NULL);
	errs	+= ai64ssc750k_ai_range			(args->fd, -1, 1, args->v_range, NULL);

	gsc_label("Settling Delay");
	os_sleep_ms(1000);
	printf("Done\n");

	errs	+= ai64ssc750k_auto_calibrate	(args->fd, -1, 1);
	errs	+= ai64ssc750k_fsamp_ai_report_all(args->fd, -1, 1, NULL);
	errs	+= ai64ssc750k_burst_enable		(args->fd, -1, 1, args->bursting, NULL);
	errs	+= ai64ssc750k_burst_size		(args->fd, -1, 1, 0, NULL);
	errs	+= ai64ssc750k_burst_src		(args->fd, -1, 1, AI64SSC750K_BURST_SRC_BCR, NULL);

	if (args->bursting == AI64SSC750K_BURST_ENABLE_YES)
		errs	+= ai64ssc750k_burst_trig	(args->fd, -1, 1);

	errs	+= ai64ssc750k_rx_io_overflow	(args->fd, -1, 1, AI64SSC750K_IO_OVERFLOW_IGNORE, NULL);
	errs	+= ai64ssc750k_rx_io_underflow	(args->fd, -1, 1, AI64SSC750K_IO_UNDERFLOW_IGNORE, NULL);
	errs	+= ai64ssc750k_ai_buf_clear		(args->fd, -1, 1);
	errs	+= ai64ssc750k_ai_buf_overflow	(args->fd, -1, 1, AI64SSC750K_AI_BUF_OVERFLOW_CLEAR, NULL);
	errs	+= ai64ssc750k_ai_buf_underflow	(args->fd, -1, 1, AI64SSC750K_AI_BUF_UNDERFLOW_CLEAR, NULL);
	errs	+= ai64ssc750k_rx_io_timeout	(args->fd, -1, 1, 300, NULL);

	gsc_label_level_dec();
	return(errs);
}



//*****************************************************************************
int perform_tests(const args_t* args)
{
	s32	chans	= 32;
	int	errs	= 0;

	gsc_label("Capture & Save");
	printf("\n");
	gsc_label_level_inc();

	errs	+= _config(args);
	errs	+= _channels					(args, &chans);
	errs	+= ai64ssc750k_ai_buf_clear		(args->fd, -1, 1);
	errs	+= _read_data(args, chans);
	errs	+= ai64ssc750k_ai_buf_overflow	(args->fd, -1, 1, AI64SSC750K_AI_BUF_OVERFLOW_CHECK, NULL);
	errs	+= ai64ssc750k_ai_buf_underflow	(args->fd, -1, 1, AI64SSC750K_AI_BUF_UNDERFLOW_CHECK, NULL);
	errs	+= _save_data(args, chans, errs);

	gsc_label_level_dec();
	return(errs);
}



