// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/18AI64SSC750K/utils/util_ai_buf_level.c $
// $Rev: 53322 $
// $Date: 2023-07-05 16:08:44 -0500 (Wed, 05 Jul 2023) $

// 18AI64SSC750K: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	ai64ssc750k_ai_buf_level
*
*	Purpose:
*
*		Provide a visual wrapper for the AI64SSC750K_IOCTL_AI_BUF_LEVEL service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		get		The current setting is recorded here, if not NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int	ai64ssc750k_ai_buf_level(int fd, int index, int verbose, s32* get)
{
	int			errs;
	const char*	ptr		= "";
	int			ret;
	s32			set;

	if (verbose)
		gsc_label_index("Input Buffer Level", index);

	ret		= ai64ssc750k_ioctl(fd, AI64SSC750K_IOCTL_AI_BUF_LEVEL, &set);
	errs	= ret ? 1 : 0;

	if (verbose)
	{
		if (set == -1)
			ptr	= ", This service is not supported by this device.";

		printf("%s  (", errs ? "FAIL <---" : "PASS");
		gsc_label_long_comma(set);
		printf(" 32-bit data items%s)\n", ptr);
	}

	if (get)
		get[0]	= set;

	return(errs);
}


