// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/18AI64SSC750K/utils/util_config_ai.c $
// $Rev: 53322 $
// $Date: 2023-07-05 16:08:44 -0500 (Wed, 05 Jul 2023) $

// 18AI64SSC750K: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	ai64ssc750k_config_ai
*
*	Purpose:
*
*		Configure the Analog Input services.
*
*	Arguments:
*
*		fd		The handle to use to access the driver.
*
*		index	The index of the device to access. Ignore if < 0.
*
*		verbose	Work verbosely?
*
*		fsamp	The desired sample rate value.
*
*	Returned:
*
*		>= 0	The number of errors encountered here.
*
******************************************************************************/

int	ai64ssc750k_config_ai(int fd, int index, int verbose, s32 fsamp)
{
	s32	chans;
	s32	enable_a;
	s32	enable_b;
	int	errs		= 0;
	s32	nrate_a;
	s32	nrate_b;
	s32	qty			= 32;
	s32	src;
	s32	src_b;
	s32	v_high;
	s32	v_out;

	if (verbose)
	{
		gsc_label_index("Input Configuration", index);
		printf("\n");
		gsc_label_level_inc();
	}

	errs	+= ai64ssc750k_initialize			(fd, index, verbose);

	// Settings must be applied after the initialization call!

	errs	+= ai64ssc750k_rx_io_mode			(fd, index, verbose, GSC_IO_MODE_PIO,					NULL);
	errs	+= ai64ssc750k_rx_io_overflow		(fd, index, verbose, AI64SSC750K_IO_OVERFLOW_CHECK,		NULL);
	errs	+= ai64ssc750k_rx_io_timeout		(fd, index, verbose, AI64SSC750K_IO_TIMEOUT_DEFAULT,	NULL);
	errs	+= ai64ssc750k_rx_io_underflow		(fd, index, verbose, AI64SSC750K_IO_UNDERFLOW_CHECK,	NULL);

	errs	+= ai64ssc750k_query				(fd, index, verbose, AI64SSC750K_QUERY_V_RANGE_HIGH, &v_high);

	if (v_high)
		v_out	= AI64SSC750K_AI_RANGE_10V;
	else
		v_out	= AI64SSC750K_AI_RANGE_5V;

	errs	+= ai64ssc750k_ai_range				(fd, index, verbose, v_out,								NULL);

	errs	+= ai64ssc750k_ai_buf_thr_lvl		(fd, index, verbose, 250000,							NULL);
	errs	+= ai64ssc750k_ai_mode				(fd, index, verbose, AI64SSC750K_AI_MODE_SINGLE,		NULL);
	errs	+= ai64ssc750k_burst_size			(fd, index, verbose, 1,									NULL);
	errs	+= ai64ssc750k_burst_src			(fd, index, verbose, AI64SSC750K_BURST_SRC_BCR,			NULL);
	errs	+= ai64ssc750k_burst_enable			(fd, index, verbose, AI64SSC750K_BURST_ENABLE_NO,		NULL);

	errs	+= ai64ssc750k_query				(fd, index, verbose, AI64SSC750K_QUERY_CHANNEL_QTY, &qty);

	if (qty > 32)
		chans	= AI64SSC750K_CHAN_ACTIVE_0_63;
	else
		chans	= AI64SSC750K_CHAN_ACTIVE_0_31;

	errs	+= ai64ssc750k_chan_active			(fd, index, verbose, chans,								NULL);
	errs	+= ai64ssc750k_chan_first			(fd, index, verbose, 0,									NULL);

	errs	+= ai64ssc750k_chan_last			(fd, index, verbose, qty - 1,							NULL);

	errs	+= ai64ssc750k_chan_single			(fd, index, verbose, 0,									NULL);
	errs	+= ai64ssc750k_data_format			(fd, index, verbose, AI64SSC750K_DATA_FORMAT_OFF_BIN,	NULL);
	errs	+= ai64ssc750k_data_packing			(fd, index, verbose, AI64SSC750K_DATA_PACKING_DISABLE,	NULL);
	errs	+= ai64ssc750k_data_width			(fd, index, verbose, AI64SSC750K_DATA_WIDTH_18_BITS,	NULL);
	errs	+= ai64ssc750k_clock_out			(fd, index, verbose, AI64SSC750K_CLOCK_OUT_DISABLE,		NULL);
	errs	+= ai64ssc750k_trigger_out			(fd, index, verbose, AI64SSC750K_TRIGGER_OUT_DISABLE,	NULL);
	errs	+= ai64ssc750k_ai_63_mode			(fd, index, verbose, AI64SSC750K_AI_63_MODE_AI_63,		NULL);
	errs	+= ai64ssc750k_scan_marker			(fd, index, verbose, AI64SSC750K_SCAN_MARKER_DISABLE,	NULL);
	errs	+= ai64ssc750k_scan_marker_set		(fd, index, verbose, 0);
	errs	+= ai64ssc750k_fsamp_ai_compute		(fd, index, verbose, fsamp, &src, &src_b, &nrate_a, &nrate_b, &enable_a, &enable_b, NULL);
	errs	+= ai64ssc750k_samp_clk_src			(fd, index, verbose, src,								NULL);
	errs	+= ai64ssc750k_rbg_clk_src			(fd, index, verbose, src_b,								NULL);
	errs	+= ai64ssc750k_rag_nrate			(fd, index, verbose, nrate_a,							NULL);
	errs	+= ai64ssc750k_rbg_nrate			(fd, index, verbose, nrate_b,							NULL);
	errs	+= ai64ssc750k_rag_enable			(fd, index, verbose, enable_a,							NULL);
	errs	+= ai64ssc750k_rbg_enable			(fd, index, verbose, enable_b,							NULL);
	errs	+= ai64ssc750k_clocking_enable		(fd, index, verbose, AI64SSC750K_CLOCKING_ENABLE_YES,	NULL);

	errs	+= ai64ssc750k_fsamp_ai_report_all	(fd, index, verbose,									NULL);

	// Settings affecting the sample rate or voltage range must generally
	// be applied before autocalibration.
	// Refer to the device user manual for clarification.
	errs	+= ai64ssc750k_autocal				(fd, index, verbose);

	// Interrupt selections should be applied after autocalibration as the
	// driver overwrites the current selection to detect the Autocalibration
	// Done interrupt. If an application intends to detect the Autocalibration
	// Done interrupt, then the interrupt selection may be made beforehand.
	errs	+= ai64ssc750k_irq0_sel				(fd, index, verbose, AI64SSC750K_IRQ0_INIT_DONE,		NULL);
	errs	+= ai64ssc750k_irq1_sel				(fd, index, verbose, AI64SSC750K_IRQ1_NONE,				NULL);

	// The final step is to clear the buffer. This service clears the FIFO and
	// resets the overflow and underflow status bits.
	errs	+= ai64ssc750k_ai_buf_clear			(fd, index, verbose);

	if (verbose)
		gsc_label_level_dec();

	return(errs);
}


