// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/18AI64SSC750K/utils/util_data_packing.c $
// $Rev: 53322 $
// $Date: 2023-07-05 16:08:44 -0500 (Wed, 05 Jul 2023) $

// 18AI64SSC750K: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	ai64ssc750k_data_packing
*
*	Purpose:
*
*		Provide a visual wrapper for the AI64SSC750K_IOCTL_DATA_PACKING service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		set		The setting to apply.
*
*		get		The current setting is recorded here, if not NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int	ai64ssc750k_data_packing(int fd, int index, int verbose, s32 set, s32* get)
{
	char		buf[128];
	int			errs;
	const char*	ptr;
	int			ret;

	if (verbose)
		gsc_label_index("Data Packing", index);

	ret		= ai64ssc750k_ioctl(fd, AI64SSC750K_IOCTL_DATA_PACKING, &set);
	errs	= ret ? 1 : 0;

	switch (set)
	{
		default:

			errs++;
			ptr	= buf;
			sprintf(buf, "Unrecognized option: 0x%lX", (long) set);
			break;

		case -1:

			ptr	= "This service is not supported by this device.";
			break;

		case AI64SSC750K_DATA_PACKING_DISABLE:

			ptr	= "Disabled";
			break;

		case AI64SSC750K_DATA_PACKING_ENABLE:

			ptr	= "Enabled";
	}

	if (verbose)
		printf("%s  (%s)\n", errs ? "FAIL <---" : "PASS", ptr);

	if (get)
		get[0]	= set;

	return(errs);
}


