// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/18AI64SSC750K/utils/util_query.c $
// $Rev: 53322 $
// $Date: 2023-07-05 16:08:44 -0500 (Wed, 05 Jul 2023) $

// 18AI64SSC750K: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	ai64ssc750k_query
*
*	Purpose:
*
*		Provide a visual wrapper for the AI64SSC750K_IOCTL_QUERY service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		set		This is the query option to access.
*
*		get		The results are reported here. This may be NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int	ai64ssc750k_query(int fd, int index, int verbose, s32 set, s32* get)
{
	char		buf[128];
	int			errs	= 0;
	const char*	ptr;
	s32			query	= set;
	int			ret;
	s32			tmp;

	switch (query)
	{
		default:

			errs++;
			ptr	= buf;
			sprintf(buf, "Query Error (#%ld)", (long) query);
			break;

		case AI64SSC750K_QUERY_AUTOCAL_MS:		ptr	= "Autocal Duration";	break;
		case AI64SSC750K_QUERY_B33_FUNC:		ptr	= "B33 Function";		break;
		case AI64SSC750K_QUERY_CHANNEL_MAX:		ptr	= "Max Channels";		break;
		case AI64SSC750K_QUERY_CHANNEL_QTY:		ptr	= "Installed Channels";	break;
		case AI64SSC750K_QUERY_COUNT:			ptr	= "Query Options";		break;
		case AI64SSC750K_QUERY_DEVICE_TYPE:		ptr	= "Device Type";		break;
		case AI64SSC750K_QUERY_FIFO_SIZE:		ptr	= "FIFO Size";			break;
		case AI64SSC750K_QUERY_FSAMP_MAX:		ptr	= "Fsamp Max";			break;
		case AI64SSC750K_QUERY_FSAMP_MIN:		ptr	= "Fsamp Min";			break;
		case AI64SSC750K_QUERY_INIT_MS:			ptr	= "Init. Duration";		break;
		case AI64SSC750K_QUERY_INPUT_BANDWIDTH:	ptr	= "AI Bandwidth";		break;
		case AI64SSC750K_QUERY_MASTER_CLOCK:	ptr	= "Master Clock";		break;
		case AI64SSC750K_QUERY_NRATE_MAX:		ptr	= "Nrate Max";			break;
		case AI64SSC750K_QUERY_NRATE_MIN:		ptr	= "Nrate Min";			break;
		case AI64SSC750K_QUERY_RATE_GEN_QTY:	ptr	= "Rate Gen Qty";		break;
		case AI64SSC750K_QUERY_RX_BUFFER_SIZE:	ptr	= "Rx Buffer Size";		break;
		case AI64SSC750K_QUERY_V_RANGE_HIGH:	ptr	= "High Voltage Range";	break;
	}

	if (verbose)
		gsc_label_index(ptr, index);

	ret		= ai64ssc750k_ioctl(fd, AI64SSC750K_IOCTL_QUERY, &set);
	errs	+= ret ? 1 : 0;

	switch (query)
	{
		default:

			errs++;
			sprintf(buf, "%ld", (long) set);
			break;

		case AI64SSC750K_QUERY_CHANNEL_MAX:
		case AI64SSC750K_QUERY_CHANNEL_QTY:
		case AI64SSC750K_QUERY_COUNT:
		case AI64SSC750K_QUERY_NRATE_MAX:
		case AI64SSC750K_QUERY_NRATE_MIN:
		case AI64SSC750K_QUERY_RATE_GEN_QTY:

			sprintf(buf, "%ld", (long) set);
			break;

		case AI64SSC750K_QUERY_INPUT_BANDWIDTH:
		case AI64SSC750K_QUERY_MASTER_CLOCK:

			sprintf(buf, "%ld Hz", (long) set);
			break;

		case AI64SSC750K_QUERY_AUTOCAL_MS:
		case AI64SSC750K_QUERY_INIT_MS:

			sprintf(buf, "%ld ms", (long) set);
			break;

		case AI64SSC750K_QUERY_FSAMP_MAX:
		case AI64SSC750K_QUERY_FSAMP_MIN:

			sprintf(buf, "%ld S/S", (long) set);
			break;

		case AI64SSC750K_QUERY_V_RANGE_HIGH:

			strcpy(buf, set ? "Yes" : "No");
			break;

		case AI64SSC750K_QUERY_RX_BUFFER_SIZE:

			ptr	= "";
			tmp	= set;

			if ((tmp) && ((tmp % 1024) == 0))
			{
				ptr	= "K";
				tmp	/= 1024;
			}

			sprintf(buf, "%ld%s Bytes", (long) tmp, ptr);
			break;

		case AI64SSC750K_QUERY_FIFO_SIZE:

			ptr	= "";
			tmp	= set;

			if ((tmp) && ((tmp % 1024) == 0))
			{
				ptr	= "K";
				tmp	/= 1024;
			}

			sprintf(buf, "%ld%s Words", (long) tmp, ptr);
			break;

		case AI64SSC750K_QUERY_DEVICE_TYPE:

			if (set == GSC_DEV_TYPE_18AI64SSC750K)
			{
				strcpy(buf, "18AI64SSC750K");
			}
			else
			{
				errs++;
				sprintf(buf, "INVALID: %ld", (long) set);
			}

			break;

		case AI64SSC750K_QUERY_B33_FUNC:

			switch (set)
			{
				default:

					errs++;
					sprintf(buf, "Invalid option: %ld", (long) set);
					break;

				case AI64SSC750K_B33_FUNC_IN_RET:	strcpy(buf, "Input Return");	break;
				case AI64SSC750K_B33_FUNC_CLK_IO:	strcpy(buf, "Clock I/O");		break;
			}

			break;
	}

	if (verbose)
	{
		if (errs)
			printf("FAIL <---  (%s)\n", buf);
		else
			printf("%s\n", buf);
	}

	if (get)
		get[0]	= set;

	return(errs);
}


